@@ -5,20 +5,21 @@ import os
 from os.path import join, getmtime, exists
 
 from .generator import PariFunctionGenerator
-from .parser import pari_share
+from .paths import pari_share
 
 
 def rebuild(force=False):
     pari_module_path = 'cypari2'
     src_files = [join(pari_share(), b'pari.desc')] + \
-                 glob.glob(join('autogen', 'pari', '*.py'))
-    gen_files = [join(pari_module_path, 'auto_gen.pxi')]
+                 glob.glob(join('autogen', '*.py'))
+    gen_files = [join(pari_module_path, 'auto_paridecl.pxd'),
+                 join(pari_module_path, 'auto_gen.pxi')]
 
-    if all(exists(f) for f in gen_files):
+    if not force and all(exists(f) for f in gen_files):
         src_mtime = max(getmtime(f) for f in src_files)
         gen_mtime = min(getmtime(f) for f in gen_files)
 
-        if gen_mtime > src_mtime and not force:
+        if gen_mtime > src_mtime:
             return
 
     G = PariFunctionGenerator()
