@@ -71,6 +71,14 @@ class PariArgument(object):
         """
         return "(generic)"
 
+    def ctype(self):
+        """
+        The corresponding C type. This is used for auto-generating
+        the declarations of the C function. In some cases, this is also
+        used for passing the argument from Python to Cython.
+        """
+        raise NotImplementedError
+
     def always_default(self):
         """
         If this returns not ``None``, it is a value which is always
@@ -156,9 +164,6 @@ class PariArgumentClass(PariArgument):
 
     The C/Cython type is given by ``self.ctype()``.
     """
-    def ctype(self):
-        raise NotImplementedError
-
     def prototype_code(self):
         """
         Return code to appear in the prototype of the Cython wrapper.
@@ -179,11 +184,15 @@ class PariInstanceArgument(PariArgumentObject):
         PariArgument.__init__(self, iter(["self"]), None, 0)
     def _typerepr(self):
         return "Pari"
+    def ctype(self):
+        return "GEN"
 
 
 class PariArgumentGEN(PariArgumentObject):
     def _typerepr(self):
         return "GEN"
+    def ctype(self):
+        return "GEN"
     def convert_code(self):
         if self.index == 0:
             # "self" is always of type Gen, we skip the conversion
@@ -191,6 +200,13 @@ class PariArgumentGEN(PariArgumentObject):
         elif self.default is None:
             s  = "        {name} = objtogen({name})\n"
             s += "        cdef GEN {tmp} = (<Gen>{name}).g\n"
+        elif self.default is False:
+            # This is actually a required argument
+            # See parse_prototype() in parser.py why we need this
+            s  = "        if {name} is None:\n"
+            s += "            raise TypeError(\"missing required argument: '{name}'\")\n"
+            s += "        {name} = objtogen({name})\n"
+            s += "        cdef GEN {tmp} = (<Gen>{name}).g\n"
         elif self.default == "NULL":
             s  = "        cdef GEN {tmp} = {default}\n"
             s += "        if {name} is not None:\n"
@@ -212,17 +228,19 @@ class PariArgumentGEN(PariArgumentObject):
 class PariArgumentString(PariArgumentObject):
     def _typerepr(self):
         return "str"
+    def ctype(self):
+        return "char *"
     def convert_code(self):
         if self.default is None:
-            s  = "        {name} = str({name})\n"
-            s += "        cdef char* {tmp} = <bytes?>{name}\n"
+            s  = "        {name} = to_bytes({name})\n"
+            s += "        cdef char* {tmp} = <bytes> {name}\n"
         else:
             s  = "        cdef char* {tmp}\n"
             s += "        if {name} is None:\n"
             s += "            {tmp} = {default}\n"
             s += "        else:\n"
-            s += "            {name} = bytes({name})\n"
-            s += "            {tmp} = <bytes?>{name}\n"
+            s += "            {name} = to_bytes({name})\n"
+            s += "            {tmp} = <bytes> {name}\n"
         return s.format(name=self.name, tmp=self.tmpname, default=self.default)
     def call_code(self):
         return self.tmpname
@@ -230,6 +248,8 @@ class PariArgumentString(PariArgumentObject):
 class PariArgumentVariable(PariArgumentObject):
     def _typerepr(self):
         return "var"
+    def ctype(self):
+        return "long"
     def default_default(self):
         return "-1"
     def convert_code(self):
@@ -313,7 +333,7 @@ pari_arg_types = {
         'b': PariArgumentBitprec,
         'P': PariArgumentSeriesPrec,
 
-    # Codes which are known but not actually supported for Sage
+    # Codes which are known but not actually supported yet
         '&': None,
         'V': None,
         'I': None,
